@@ -181,12 +181,10 @@ impl Value {
                 .parse::<i8>()
                 .map(Value::I8)
                 .map_err(|_| ValueError::LiteralCastFromTextToIntegerFailed(v.to_string()).into()),
-            (DataType::Int8, Literal::Number(v)) => v
-                .to_f64()
-                .map(|v| Value::I8(v.trunc() as i8))
-                .ok_or_else(|| {
-                    ValueError::UnreachableLiteralCastFromNumberToInteger(v.to_string()).into()
-                }),
+            (DataType::Int8, Literal::Number(v)) => match v.to_i8() {
+                Some(x) => Ok(Value::I8(x)),
+                None => Err(ValueError::LiteralCastToInt8Failed(v.to_string()).into()),
+            },
             (DataType::Int8, Literal::Boolean(v)) => {
                 let v = if *v { 1 } else { 0 };
 
@@ -204,6 +202,21 @@ impl Value {
 
                 Ok(Value::F64(v))
             }
+            (DataType::Decimal, Literal::Text(v)) => v
+                .parse::<Decimal>()
+                .map(Value::Decimal)
+                .map_err(|_| ValueError::LiteralCastFromTextToDecimalFailed(v.to_string()).into()),
+            (DataType::Decimal, Literal::Number(v)) => v
+                .to_string()
+                .parse::<Decimal>()
+                .map(Value::Decimal)
+                .map_err(|_| ValueError::LiteralCastFromTextToDecimalFailed(v.to_string()).into()),
+            (DataType::Decimal, Literal::Boolean(v)) => {
+                let v = if *v { Decimal::ONE } else { Decimal::ZERO };
+
+                Ok(Value::Decimal(v))
+            }
+
             (DataType::Text, Literal::Number(v)) => Ok(Value::Str(v.to_string())),
             (DataType::Text, Literal::Text(v)) => Ok(Value::Str(v.to_string())),
             (DataType::Text, Literal::Boolean(v)) => {
@@ -219,6 +232,7 @@ impl Value {
             | (DataType::Int, Literal::Null)
             | (DataType::Int8, Literal::Null)
             | (DataType::Float, Literal::Null)
+            | (DataType::Decimal, Literal::Null)
             | (DataType::Text, Literal::Null) => Ok(Value::Null),
             (DataType::Date, Literal::Text(v)) => parse_date(v)
                 .map(Value::Date)
@@ -349,4 +363,260 @@ mod tests {
         test!("03:30:37.123 PM", time(15, 30, 37, 123));
         test!("9:30:37.917 AM", time(9, 30, 37, 917));
     }
+
+    #[test]
+    fn try_from_literal() {
+        use {
+            crate::{ast::DataType, data::Interval as I},
+            chrono::NaiveDate,
+            rust_decimal::Decimal,
+            std::{borrow::Cow, str::FromStr},
+        };
+
+        macro_rules! num {
+            ($num: expr) => {
+                Literal::Number(Cow::Owned(BigDecimal::from_str($num).unwrap()))
+            };
+        }
+
+        macro_rules! text {
+            ($text: expr) => {
+                Literal::Text(Cow::Owned($text.to_owned()))
+            };
+        }
+
+        macro_rules! test {
+            ($to: expr, $from: expr, $expected: expr) => {
+                assert_eq!(Value::try_from_literal(&$to, &$from), Ok($expected));
+            };
+        }
+
+        let timestamp = |y, m, d, hh, mm, ss, ms| {
+            chrono::NaiveDate::from_ymd(y, m, d).and_hms_milli(hh, mm, ss, ms)
+        };
+
+        test!(DataType::Boolean, Literal::Boolean(true), Value::Bool(true));
+        test!(DataType::Int, num!("123456789"), Value::I64(123456789));
+        test!(DataType::Int8, num!("64"), Value::I8(64));
+        test!(DataType::Float, num!("123456789"), Value::F64(123456789.0));
+        test!(
+            DataType::Text,
+            text!("Good!"),
+            Value::Str("Good!".to_owned())
+        );
+        test!(
+            DataType::Date,
+            text!("2015-09-05"),
+            Value::Date(NaiveDate::from_ymd(2015, 9, 5))
+        );
+        test!(
+            DataType::Timestamp,
+            text!("2022-12-20 10:00:00.987"),
+            Value::Timestamp(timestamp(2022, 12, 20, 10, 0, 0, 987))
+        );
+        test!(
+            DataType::Time,
+            text!("12:00:35"),
+            Value::Time(chrono::NaiveTime::from_hms_milli(12, 0, 35, 0))
+        );
+        test!(
+            DataType::Interval,
+            Literal::Interval(I::Month(1)),
+            Value::Interval(I::Month(1))
+        );
+        test!(
+            DataType::Interval,
+            Literal::Interval(I::Microsecond(1234567890)),
+            Value::Interval(I::Microsecond(1234567890))
+        );
+        test!(
+            DataType::Uuid,
+            text!("936DA01F9ABD4d9d80C702AF85C822A8"),
+            Value::Uuid(195965723427462096757863453463987888808)
+        );
+        assert_eq!(
+            Value::try_from_literal(
+                &DataType::Map,
+                &text!(
+                    r#"{
+            "name": "John Doe",
+            "age": 43
+        }"#
+                )
+            ),
+            Value::parse_json_map(
+                r#"{
+            "name": "John Doe",
+            "age": 43
+        }"#
+            )
+        );
+        assert_eq!(
+            Value::try_from_literal(
+                &DataType::List,
+                &text!(
+                    r#"[
+            "+44 1234567",
+            "+44 2345678"
+        ]"#
+                )
+            ),
+            Value::parse_json_list(
+                r#"[
+            "+44 1234567",
+            "+44 2345678"
+        ]"#
+            )
+        );
+        test!(
+            DataType::Decimal,
+            num!("200"),
+            Value::Decimal(Decimal::new(200, 0))
+        );
+    }
+
+    #[test]
+    fn try_from() {
+        use {
+            crate::data::Interval as I,
+            std::{borrow::Cow, str::FromStr},
+        };
+
+        macro_rules! text {
+            ($text: expr) => {
+                Literal::Text(Cow::Owned($text.to_owned()))
+            };
+        }
+
+        macro_rules! num {
+            ($num: expr) => {
+                &Literal::Number(Cow::Owned(BigDecimal::from_str($num).unwrap()))
+            };
+        }
+
+        macro_rules! test {
+            ($from: expr, $expected: expr) => {
+                assert_eq!(Value::try_from($from), Ok($expected));
+            };
+        }
+
+        test!(text!("hello"), Value::Str("hello".to_owned()));
+        test!(&text!("hallo"), Value::Str("hallo".to_owned()));
+        test!(num!("1234567890"), Value::I64(1234567890));
+        test!(num!("12345678.90"), Value::F64(12345678.90));
+        test!(&Literal::Boolean(false), Value::Bool(false));
+        test!(
+            &Literal::Interval(I::Month(1)),
+            Value::Interval(I::Month(1))
+        );
+        assert!(matches!(Value::try_from(&Literal::Null), Ok(Value::Null)))
+    }
+
+    #[test]
+    fn try_cast_from_literal() {
+        use {
+            crate::{ast::DataType, data::Interval as I},
+            chrono::NaiveDate,
+            std::{borrow::Cow, str::FromStr},
+        };
+
+        macro_rules! text {
+            ($text: expr) => {
+                Literal::Text(Cow::Owned($text.to_owned()))
+            };
+        }
+
+        macro_rules! num {
+            ($num: expr) => {
+                &Literal::Number(Cow::Owned(BigDecimal::from_str($num).unwrap()))
+            };
+        }
+
+        macro_rules! test {
+            ($to: expr, $from: expr, $expected: expr) => {
+                assert_eq!(Value::try_cast_from_literal(&$to, &$from), Ok($expected))
+            };
+        }
+
+        macro_rules! test_null {
+            ($to: expr, $from: expr) => {
+                assert!(matches!(
+                    Value::try_cast_from_literal(&$to, &$from),
+                    Ok(Value::Null)
+                ))
+            };
+        }
+
+        let timestamp = |y, m, d, hh, mm, ss, ms| {
+            chrono::NaiveDate::from_ymd(y, m, d).and_hms_milli(hh, mm, ss, ms)
+        };
+
+        test!(
+            DataType::Boolean,
+            Literal::Boolean(false),
+            Value::Bool(false)
+        );
+        test!(DataType::Boolean, text!("false"), Value::Bool(false));
+        test!(DataType::Boolean, text!("true"), Value::Bool(true));
+        test!(DataType::Boolean, num!("0"), Value::Bool(false));
+        test!(DataType::Boolean, num!("1"), Value::Bool(true));
+        test!(DataType::Int, text!("1234567890"), Value::I64(1234567890));
+        test!(DataType::Int, num!("1234567890"), Value::I64(1234567890));
+        test!(DataType::Int, Literal::Boolean(true), Value::I64(1));
+        test!(DataType::Int, Literal::Boolean(false), Value::I64(0));
+        test!(DataType::Int8, text!("127"), Value::I8(127));
+        test!(DataType::Int8, num!("125"), Value::I8(125));
+        test!(DataType::Int8, Literal::Boolean(true), Value::I8(1));
+        test!(DataType::Int8, Literal::Boolean(false), Value::I8(0));
+        test!(DataType::Float, text!("12345.6789"), Value::F64(12345.6789));
+        test!(DataType::Float, num!("123456.789"), Value::F64(123456.789));
+        test!(DataType::Float, Literal::Boolean(true), Value::F64(1.0));
+        test!(DataType::Float, Literal::Boolean(false), Value::F64(0.0));
+        test!(
+            DataType::Text,
+            num!("1234567890"),
+            Value::Str("1234567890".to_owned())
+        );
+        test!(DataType::Text, text!("Cow"), Value::Str("Cow".to_owned()));
+        test!(
+            DataType::Text,
+            Literal::Boolean(true),
+            Value::Str("TRUE".to_owned())
+        );
+        test!(
+            DataType::Text,
+            Literal::Boolean(false),
+            Value::Str("FALSE".to_owned())
+        );
+        test!(
+            DataType::Interval,
+            text!(r#""+22-10" YEAR TO MONTH"#),
+            Value::Interval(I::Month(274))
+        );
+        test!(
+            DataType::Uuid,
+            text!("936DA01F9ABD4d9d80C702AF85C822A8"),
+            Value::Uuid(195965723427462096757863453463987888808)
+        );
+        test_null!(DataType::Boolean, Literal::Null);
+        test_null!(DataType::Int, Literal::Null);
+        test_null!(DataType::Int8, Literal::Null);
+        test_null!(DataType::Float, Literal::Null);
+        test_null!(DataType::Text, Literal::Null);
+        test!(
+            DataType::Date,
+            text!("2015-09-05"),
+            Value::Date(NaiveDate::from_ymd(2015, 9, 5))
+        );
+        test!(
+            DataType::Time,
+            text!("12:00:35"),
+            Value::Time(chrono::NaiveTime::from_hms_milli(12, 0, 35, 0))
+        );
+        test!(
+            DataType::Timestamp,
+            text!("2022-12-20 10:00:00.987"),
+            Value::Timestamp(timestamp(2022, 12, 20, 10, 0, 0, 987))
+        );
+    }
 }
